package eu.jonahbauer.raytracing.math;

import eu.jonahbauer.raytracing.render.spectrum.SampledWavelengths;
import eu.jonahbauer.raytracing.scene.HitResult;
import org.jetbrains.annotations.NotNull;

import java.util.Objects;

public record Ray(@NotNull Vec3 origin, @NotNull Vec3 direction, @NotNull SampledWavelengths lambda) {
    public Ray {
        Objects.requireNonNull(origin, "origin");
        Objects.requireNonNull(direction, "direction");
        Objects.requireNonNull(lambda, "lambda");
    }

    public Ray(@NotNull Vec3 origin, @NotNull Vec3 direction) {
        this(origin, direction, SampledWavelengths.EMPTY);
    }

    public @NotNull Vec3 at(double t) {
        return Vec3.fma(t, direction, origin);
    }

    public @NotNull Ray with(@NotNull HitResult hit, @NotNull Vec3 direction) {
        return new Ray(hit.position(), direction, lambda);
    }

    public @NotNull Ray with(@NotNull Vec3 origin, @NotNull Vec3 direction) {
        return new Ray(origin, direction, lambda);
    }

    public @NotNull Ray with(@NotNull SampledWavelengths lambda) {
        return new Ray(origin, direction, lambda);
    }
}
